import gc
import os
import re
import shutil
import sys
from multiprocessing import Pool

import numpy as np
import pandas as pd

import wgdi.base as base
import wgdi.collinearity as improvedcollinearity


class mycollinearity():
    def __init__(self, options):
        self.repeat_number = 10
        self.multiple = 1
        self.score = 100
        self.evalue = 1e-5
        self.blast_reverse = 'False'
        self.over_windows = 5
        self.options = options
        for k, v in options:
            setattr(self, str(k), v)
            print(str(k), ' = ', v)
        self.position = 'order'
        if hasattr(self, 'grading'):
            self.grading = [int(k) for k in self.grading.split(',')]
        else:
            self.grading = [50, 40, 25]
        if hasattr(self, 'process'):
            self.process = int(self.process)
        else:
            self.process = 4

    def deal_blast(self, blast, gff1, gff2, rednum, repeat_number):
        blast['grading'] = 0
        bluenum = 4+rednum
        index = [group.sort_values(by=[11], ascending=[False])[:repeat_number].index.tolist()
                 for name, group in blast.groupby([0])]
        reddata = np.array([k[:rednum] for k in index], dtype=object)
        bluedata = np.array([k[rednum:bluenum] for k in index], dtype=object)
        graydata = np.array([k[bluenum:repeat_number]
                             for k in index], dtype=object)
        if len(reddata):
            redindex = np.concatenate(reddata)
        else:
            redindex = []
        if len(bluedata):
            blueindex = np.concatenate(bluedata)
        else:
            blueindex = []
        if len(graydata):
            grayindex = np.concatenate(graydata)
        else:
            grayindex = []
        blast.loc[redindex, 'grading'] = self.grading[0]
        blast.loc[blueindex, 'grading'] = self.grading[1]
        blast.loc[grayindex, 'grading'] = self.grading[2]
        return blast[blast['grading'] > 0]

    def run(self):
        lens1 = base.newlens(self.lens1, 'order')
        lens2 = base.newlens(self.lens2, 'order')
        gff1 = base.newgff(self.gff1)
        gff2 = base.newgff(self.gff2)
        gff1 = gff1[gff1['chr'].isin(lens1.index)]
        gff2 = gff2[gff2['chr'].isin(lens2.index)]
        blast = base.newblast(self.blast, int(self.score), float(
            self.evalue), gff1, gff2, self.blast_reverse)
        blast = self.deal_blast(blast, gff1, gff2, int(
            self.multiple), int(self.repeat_number))
        blast['loc1'] = blast[0].map(gff1.loc[:, self.position])
        blast['loc2'] = blast[1].map(gff2.loc[:, self.position])
        blast['chr1'] = blast[0].map(gff1.loc[:, 'chr'])
        blast['chr2'] = blast[1].map(gff2.loc[:, 'chr'])
        total = []
        for (chr1, chr2), group in blast.groupby(['chr1', 'chr2']):
            total.append([chr1, chr2, group])
        del blast, group
        gc.collect()
        n = int(np.ceil(len(total) / float(self.process)))
        result, data = '', []
        try:
            pool = Pool(self.process)
            for i in range(0, len(total), n):
                data.append(pool.apply_async(self.single_pool, args=(
                    total[i:i + n], gff1, gff2, lens1, lens2)))
            pool.close()
            pool.join()
        except:
            pool.terminate()
        for k in data:
            result += k.get()
        result = re.split('\n', result)
        fout = open(self.savefile, 'w')
        num = 1
        for line in result:
            if re.match(r"# Alignment", line):
                s = '# Alignment ' + str(num) + ':'
                fout.writelines(s+line.split(':')[1]+'\n')
                num += 1
                continue
            if len(line) > 0:
                fout.writelines(line+'\n')
        fout.close()
        sys.exit(0)

    def single_pool(self, group, gff1, gff2, lens1, lens2):
        text = ''
        for bk in group:
            chr1, chr2 = str(bk[0]), str(bk[1])
            print('runing ', chr1, 'vs', chr2)
            df = pd.DataFrame(np.zeros((lens1[chr1], lens2[chr2])))
            for index, row in bk[2].iterrows():
                df.loc[row['loc1'], row['loc2']] = row['grading']
            df = df.loc[:, df.sum(axis=0) != 0]
            df = df.loc[df.sum(axis=1) != 0, :]
            collinearity = improvedcollinearity.collinearity(
                self.options, df)
            data = collinearity.run()
            gf1, gf2 = gff1[gff1['chr'] == chr1], gff2[gff2['chr'] == chr2]
            blocks, evalues, socres = data
            n = 1
            for i in range(len(blocks)):
                if len(blocks[i]) < self.over_windows:
                    continue
                if blocks[i][1][0]-blocks[i][0][0] > 0:
                    mark = 'plus'
                else:
                    mark = 'minus'
                text += '# Alignment '+str(n)+': score='+str(socres[i])+' e_value=' + str(
                    evalues[i])+' N=' + str(len(blocks[i])) + ' '+str(chr1)+'&'+str(chr2) + ' ' + mark+'\n'
                n += 1
                for k in blocks[i]:
                    name1 = gf1[gf1['order'] == k[0]].index[0]
                    name2 = gf2[gf2['order'] == k[1]].index[0]
                    if gf1.loc[name1, 'stand'] == gf2.loc[name2, 'stand']:
                        order = '1'
                    else:
                        order = '-1'
                    s = ' '.join([name1, str(k[0]), name2, str(k[1]), order])
                    text += s+'\n'
        return text



import re
import sys

import numpy as np
import pandas as pd
import wgdi.base as base


class block_correspondence():
    def __init__(self, options):
        self.tandem = True
        self.pvalue = 0.01
        self.position = 'order'
        self.block_length = 5
        self.tandem_length = 200
        for k, v in options:
            setattr(self, str(k), v)
            print(k, ' = ', v)
        self.homo = [float(k) for k in self.homo.split(',')]

    def run(self):
        lens1 = base.newlens(self.lens1, self.position)
        lens2 = base.newlens(self.lens2, self.position)
        bkinfo = pd.read_csv(self.blockinfo)
        bkinfo['chr1'] = bkinfo['chr1'].astype(str)
        bkinfo['chr2'] = bkinfo['chr2'].astype(str)
        bkinfo = bkinfo[(bkinfo['length'] >= int(self.block_length)) & (bkinfo['chr1'].isin(
            lens1.index)) & (bkinfo['chr2'].isin(lens2.index)) & (bkinfo['pvalue'] <= float(self.pvalue))]
        cor = [[k, i, 0, lens1[i], j, 0, lens2[j], float(self.homo[0]), float(self.homo[1])] for k in range(
            1, int(self.multiple)+1) for i in lens1.index for j in lens2.index]
        cor = pd.DataFrame(
            cor, columns=['sub', 'chr1', 'start1', 'end1', 'chr2', 'start2', 'end2', 'homo1', 'homo2'])
        cor['chr1'] = cor['chr1'].astype(str)
        cor['chr2'] = cor['chr2'].astype(str)
        if self.tandem == True or self.tandem == 'true' or self.tandem == 1:
            bkinfo = self.remove_tandem(bkinfo)
        arr = self.collinearity_region(cor, bkinfo, lens1)
        bkinfo.loc[bkinfo.index.isin(arr), :].to_csv(
            self.savefile, index=False)

    def remove_tandem(self, bkinfo):
        group = bkinfo[bkinfo['chr1'] == bkinfo['chr2']].copy()
        group.loc[:, 'start'] = group.loc[:, 'start1']-group.loc[:, 'start2']
        group.loc[:, 'end'] = group.loc[:, 'end1']-group.loc[:, 'end2']
        index = group[(group['start'].abs() <= int(self.tandem_length)) | (
            group['end'].abs() <= int(self.tandem_length))].index
        bkinfo = bkinfo.drop(index)
        return bkinfo

    def collinearity_region(self, cor, bkinfo,lens):
        arr = []
        for (chr1,chr2), group in bkinfo.groupby(['chr1','chr2']):
            group = group.sort_values(by=['length'], ascending=[True])
            # print(chr1)
            # print(lens[str(chr1)])
            df = pd.Series(0,index=range(1,int(lens[str(chr1)])+1))
            for index, row in group.iterrows():
                if row['homo'+self.multiple] < float(self.homo[0]) or row['homo'+self.multiple] > float(self.homo[1]):
                    continue
                b1 = row['block1'].split('_')
                df1 = df.copy() 
                df1[[int(k) for k in b1]]+=1
                ratio = (len(df1[df1>0])-len(df[df>0]))/len(b1)
                # print(len(df1[df1>0]),len(df[df>0]),len(b1),ratio)
                if ratio <0.5:
                    print(ratio)
                    continue
                df[[int(k) for k in b1]]+=1
                arr.append(index)
        return arr



import re
import sys
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
import wgdi.base as base


class block_ks():
    def __init__(self, options):
        self.markersize = 0.8
        self.figsize = 'default'
        self.tandem_length = 200
        self.tandem = False
        self.area = [0, 3]
        self.position = 'order'
        self.ks_col = 'ks_NG86'
        self.pvalue = 0.01
        for k, v in options:
            setattr(self, str(k), v)
            print(str(k), ' = ', v)
        self.area = [float(k) for k in self.area.split(',')]
        if not hasattr(self, 'blockinfo_reverse'):
            self.blockinfo_reverse = 'false'

    def block_position(self, bkinfo, lens1, lens2, step1, step2):
        pos, pairs = [], []
        dict_y_chr = dict(zip(lens1.index, np.append(
            np.array([0]), lens1.cumsum()[:-1].values)))
        dict_x_chr = dict(zip(lens2.index, np.append(
            np.array([0]), lens2.cumsum()[:-1].values)))
        for index, row in bkinfo.iterrows():
            block1 = row['block1'].split('_')
            block2 = row['block2'].split('_')
            ks = row['ks'].split('_')
            locy_median = (dict_y_chr[row['chr1']] +
                           0.5*(row['end1']+row['start1']))*step1
            locx_median = (dict_x_chr[row['chr2']] +
                           0.5*(row['end2']+row['start2']))*step2
            pos.append([locx_median, locy_median, row['ks_median']])
            if len(block1) != len(ks):
                print(row['ks'])
                print(row['id'], len(block1), len(ks))
            for i in range(len(block1)):
                locy = (dict_y_chr[row['chr1']]+float(block1[i]))*step1
                locx = (dict_x_chr[row['chr2']]+float(block2[i]))*step2
                pairs.append([locx, locy, float(ks[i])])
        return pos, pairs

    def remove_tandem(self, bkinfo):
        group = bkinfo[bkinfo['chr1'] == bkinfo['chr2']].copy()
        group.loc[:, 'start'] = group.loc[:, 'start1']-group.loc[:, 'start2']
        group.loc[:, 'end'] = group.loc[:, 'end1']-group.loc[:, 'end2']
        index = group[(group['start'].abs() <= int(self.tandem_length)) | (
            group['end'].abs() <= int(self.tandem_length))].index
        bkinfo = bkinfo.drop(index)
        return bkinfo

    def run(self):
        axis = [0, 1, 1, 0]
        lens1 = base.newlens(self.lens1, self.position)
        lens2 = base.newlens(self.lens2, self.position)
        if re.search('\d', self.figsize):
            self.figsize = [float(k) for k in self.figsize.split(',')]
        else:
            self.figsize = np.array(
                [1, float(lens1.sum())/float(lens2.sum())])*10
        step1 = 1 / float(lens1.sum())
        step2 = 1 / float(lens2.sum())
        fig, ax = plt.subplots(figsize=self.figsize)
        plt.rcParams['ytick.major.pad'] = 0
        ax.xaxis.set_ticks_position('top')
        base.dotplot_frame(fig, ax, lens1, lens2, step1, step2,
                           self.genome1_name, self.genome2_name, [1, 1])
        bkinfo = pd.read_csv(self.blockinfo)
        if self.blockinfo_reverse == True or self.blockinfo_reverse.upper() == 'TRUE':
            bkinfo[['chr1', 'chr2']] = bkinfo[['chr2', 'chr1']]
            bkinfo[['block1', 'block2']] = bkinfo[['block2', 'block1']]
        bkinfo['chr1'] = bkinfo['chr1'].astype(str)
        bkinfo['chr2'] = bkinfo['chr2'].astype(str)
        bkinfo = bkinfo[(bkinfo['length'] > int(self.block_length)) & (bkinfo['chr1'].isin(
            lens1.index)) & (bkinfo['chr2'].isin(lens2.index)) & (bkinfo['pvalue'] < float(self.pvalue))]
        if self.tandem == True or self.tandem == 'true' or self.tandem == 1:
            bkinfo = self.remove_tandem(bkinfo)
        pos, pairs = self.block_position(bkinfo, lens1, lens2, step1, step2)
        cm = plt.cm.get_cmap('gist_rainbow')  # gist_rainbow,hsv
        df = pd.DataFrame(pairs, columns=['loc1', 'loc2', 'ks'])
        df = df[(df['ks'] >= self.area[0]) & (df['ks'] <= self.area[1])]
        df.drop_duplicates(inplace=True)
        sc = plt.scatter(df['loc1'], df['loc2'], s=float(self.markersize), c=df['ks'],
                         alpha=0.9, edgecolors=None, linewidths=0, marker='o', vmin=self.area[0], vmax=self.area[1], cmap=cm)
        # sc = plt.scatter(df['loc1'], df['loc2'], s=float(self.markersize), c='dimgray',
        #                  alpha=0.7, edgecolors=None, linewidths=0, marker='o', vmin=self.area[0], vmax=self.area[1])
        cbar = fig.colorbar(sc, shrink=0.5, pad=0.03, fraction=0.1)
        align = dict(family='Arial', style='normal',
                     horizontalalignment="center", verticalalignment="center")
        cbar.set_label('Ks', labelpad=12.5, fontsize=18, **align)
        ax.axis(axis)
        plt.subplots_adjust(left=0.09, right=0.96, top=0.93, bottom=0.03)
        plt.savefig(self.savefig, dpi=500)
        plt.show()
        sys.exit(0)



import re
import sys

import matplotlib.patches as mpatches
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
import wgdi.base as base


class align_dotplot():
    def __init__(self, options):
        self.position = 'order'
        self.figsize = 'default'
        for k, v in options:
            setattr(self, str(k), v)
            print(str(k), ' = ', v)
        if hasattr(self, 'ks_area'):
            self.ks_area = [float(k) for k in self.ks_area.split(',')]
        else:
            self.ks_area = [-1, 3]
        if hasattr(self, 'colors'):
            self.colors = [str(k) for k in self.colors.split(',')]
        else:
            self.colors = ['red', 'blue', 'green', 'black', 'orange']
        if not hasattr(self, 'blockinfo_reverse'):
            self.blockinfo_reverse = 'false'

    def pair_positon(self, alignment, loc1, loc2, colors):
        alignment.index = alignment.index.map(loc1)
        data, i = [], 0
        for k in alignment.columns:
            df = alignment[k].map(loc2)
            df.dropna(axis=0, how='any', inplace=True)
            for index, row in df.iteritems():
                data.append([index, row, colors[i]])
            i += 1
        df = pd.DataFrame(data, columns=['loc1', 'loc2', 'color'])
        return df

    def run(self):
        axis = [0, 1, 1, 0]
        lens1 = base.newlens(self.lens1, self.position)
        lens2 = base.newlens(self.lens2, self.position)
        if re.search('\d', self.figsize):
            self.figsize = [float(k) for k in self.figsize.split(',')]
        else:
            self.figsize = np.array(
                [1, float(lens1.sum())/float(lens2.sum())])*10
        plt.rcParams['ytick.major.pad'] = 0
        fig, ax = plt.subplots(figsize=self.figsize)
        ax.xaxis.set_ticks_position('top')
        step1 = 1 / float(lens1.sum())
        step2 = 1 / float(lens2.sum())
        base.dotplot_frame(fig, ax, lens1, lens2, step1, step2,
                           self.genome1_name, self.genome2_name, [0, 1])
        gff1 = base.newgff(self.gff1)
        gff2 = base.newgff(self.gff2)
        gff1 = base.gene_location(gff1, lens1, step1, self.position)
        gff2 = base.gene_location(gff2, lens2, step2, self.position)
        bkinfo = pd.read_csv(self.blockinfo, index_col='id')
        if self.blockinfo_reverse == True or self.blockinfo_reverse.upper() == 'TRUE':
            bkinfo[['chr1', 'chr2']] = bkinfo[['chr2', 'chr1']]
            bkinfo[['block1', 'block2']] = bkinfo[['block2', 'block1']]
        bkinfo['chr1'] = bkinfo['chr1'].astype(str)
        bkinfo['chr2'] = bkinfo['chr2'].astype(str)
        bkinfo['class'] = bkinfo['class'].astype(str)
        bkinfo=bkinfo[bkinfo['chr1'].isin(lens1.index) & (bkinfo['chr2'].isin(lens2.index))]
        align = self.alignment(gff1, gff2, bkinfo)
        alignment = align[gff1.columns[-int(
            len(bkinfo['class'].drop_duplicates())):]]
        alignment.to_csv(self.savefile, header=None)
        df = self.pair_positon(
            alignment, gff1['loc'], gff2['loc'], self.colors)
        plt.scatter(df['loc2'], df['loc1'], s=float(self.markersize), c=df['color'],
                    alpha=0.5, edgecolors=None, linewidths=0, marker='o')
        ax.axis(axis)
        plt.subplots_adjust(left=0.07, right=0.97, top=0.93, bottom=0.03)
        plt.savefig(self.savefig, dpi=500)
        plt.show()
        sys.exit(0)

    def alignment(self, gff1, gff2, bkinfo):
        for cl, group in bkinfo.groupby(['class']):
            name = 'l'+cl
            gff1[name] = np.nan
            group = group.sort_values(by=['length'], ascending=[True])
            for index, row in group.iterrows():
                b1 = row['block1'].split('_')
                b2 = row['block2'].split('_')
                ks = row['ks'].split('_')
                ks = list(map(float, ks))
                block1, block2 = [], []
                for i in range(len(ks)):
                    if self.ks_area[0] <= ks[i] <= self.ks_area[1]:
                        block1.append(int(b1[i]))
                        block2.append(int(b2[i]))
                block1 = list(map(int, block1))
                block2 = list(map(int, block2))
                area = gff1[(gff1['chr'] == row['chr1']) & (
                    gff1['order'] >= min(block1)) & (gff1['order'] <= max(block1))].index
                index1 = gff1[(gff1['chr'] == row['chr1']) & (gff1['order'].isin(
                    block1))].sort_values(by=['order'], key=lambda x: block1).index
                index2 = gff2[(gff2['chr'] == row['chr2']) & (gff2['order'].isin(
                    block2))].sort_values(by=['order'], key=lambda x: block2).index
                gff1.loc[index1, name] = index2
                gff1.loc[gff1.index.isin(area) & gff1[name].isna(), name] = '.'
        return gff1



import numpy as np
import pandas as pd
import wgdi.base as base


class block_info():
    def __init__(self, options):
        self.repeat_number = 20
        self.ks_col = 'ks_NG86'
        self.blast_reverse = 'False'
        for k, v in options:
            setattr(self, str(k), v)
            print(str(k), ' = ', v)

    def block_position(self, collinearity, blast, gff1, gff2, ks):
        data = []
        for block in collinearity:
            blk_homo, blk_ks = [],  []
            if block[1][0][0] not in gff1.index or block[1][0][2] not in gff2.index:
                continue
            chr1, chr2 = gff1.loc[block[1][0][0],
                                  'chr'], gff2.loc[block[1][0][2], 'chr']
            array1, array2 = [float(i[1]) for i in block[1]], [
                float(i[3]) for i in block[1]]
            start1, end1 = array1[0], array1[-1]
            start2, end2 = array2[0], array2[-1]
            block1, block2 = [], []
            for k in block[1]:
                block1.append(int(float(k[1])))
                block2.append(int(float(k[3])))
                if k[0]+","+k[2] in ks.index:
                    pair_ks = ks[str(k[0])+","+str(k[2])]
                    blk_ks.append(pair_ks)
                elif k[2]+","+k[0] in ks.index:
                    pair_ks = ks[str(k[2])+","+str(k[0])]
                    blk_ks.append(pair_ks)
                else:
                    blk_ks.append(-1)
                if k[0]+","+k[2] not in blast.index:
                    continue
                blk_homo.append(
                    blast.loc[k[0]+","+k[2], ['homo'+str(i) for i in range(1, 6)]].values.tolist())
            ks_arr = [k for k in blk_ks if k >= 0]
            if len(ks_arr) == 0:
                ks_median = -1
                ks_average = -1
            else:
                arr_ks = [k for k in blk_ks if k >= 0]
                ks_median = base.get_median(arr_ks)
                ks_average = sum(arr_ks)/len(arr_ks)
            df = pd.DataFrame(blk_homo)
            homo = df.mean().values
            if len(homo) == 0:
                homo = [-1, -1, -1, -1, -1]
            blkks = '_'.join([str(k) for k in blk_ks])
            block1 = '_'.join([str(k) for k in block1])
            block2 = '_'.join([str(k) for k in block2])
            data.append([block[0], chr1, chr2, start1, end1, start2, end2, block[2], len(
                block[1]), ks_median, ks_average, homo[0], homo[1], homo[2], homo[3], homo[4], block1, block2, blkks])
        data = pd.DataFrame(data, columns=['id', 'chr1', 'chr2', 'start1', 'end1', 'start2', 'end2',
                                           'pvalue', 'length', 'ks_median', 'ks_average', 'homo1', 'homo2', 'homo3',
                                           'homo4', 'homo5', 'block1', 'block2', 'ks'])
        data['density1'] = data['length'] / \
            ((data['end1']-data['start1']).abs()+1)
        data['density2'] = data['length'] / \
            ((data['end2']-data['start2']).abs()+1)
        return data

    def remove_tandem(self, bkinfo):
        group = bkinfo[bkinfo['chr1'] == bkinfo['chr2']].copy()
        group.loc[:, 'start'] = group.loc[:, 'start1']-group.loc[:, 'start2']
        group.loc[:, 'end'] = group.loc[:, 'end1']-group.loc[:, 'end2']
        index = group[(group['start'].abs() <= int(self.tandem_length)) | (
            group['end'].abs() <= int(self.tandem_length))].index
        bkinfo = bkinfo.drop(index)
        return bkinfo

    def blast_homo(self, blast, gff1, gff2, repeat_number):
        index = [group.sort_values(by=11, ascending=False)[:repeat_number].index.tolist()
                 for name, group in blast.groupby([0])]
        blast = blast.loc[np.concatenate(
            np.array([k[:repeat_number] for k in index], dtype=object)), [0, 1]]
        blast = blast.assign(homo1=np.nan, homo2=np.nan,
                             homo3=np.nan, homo4=np.nan, homo5=np.nan)
        for i in range(1, 6):
            bluenum = i+5
            redindex = np.concatenate(
                np.array([k[:i] for k in index], dtype=object))
            blueindex = np.concatenate(
                np.array([k[i:bluenum] for k in index], dtype=object))
            grayindex = np.concatenate(
                np.array([k[bluenum:repeat_number] for k in index], dtype=object))
            blast.loc[redindex, 'homo'+str(i)] = 1
            blast.loc[blueindex, 'homo'+str(i)] = 0
            blast.loc[grayindex, 'homo'+str(i)] = -1
        return blast

    def run(self):
        lens1 = base.newlens(self.lens1, self.position)
        lens2 = base.newlens(self.lens2, self.position)
        gff1 = base.newgff(self.gff1)
        gff2 = base.newgff(self.gff2)
        gff1 = gff1[gff1['chr'].isin(lens1.index)]
        gff2 = gff2[gff2['chr'].isin(lens2.index)]
        blast = base.newblast(self.blast, int(self.score), float(
            self.evalue), gff1, gff2, self.blast_reverse)
        blast = self.blast_homo(blast, gff1, gff2, int(self.repeat_number))
        blast.index = blast[0]+','+blast[1]
        collinearity = self.auto_file(gff1, gff2)
        ks = base.read_ks(self.ks, self.ks_col)
        data = self.block_position(collinearity, blast, gff1, gff2, ks)
        data['class1'] = 0
        data['class2'] = 0
        data.to_csv(self.savefile, index=None)

    def auto_file(self, gff1, gff2):
        p = pd.read_csv(self.collinearity, sep='\n', header=None, nrows=30)
        p = '\n'.join(p[0])
        if 'path length' in p or 'MAXIMUM GAP' in p:
            collinearity = base.read_colinearscan(self.collinearity)
        elif 'MATCH_SIZE' in p or '## Alignment' in p:
            col = base.read_mcscanx(self.collinearity)
            collinearity = []
            for block in col:
                newblock = []
                for k in block[1]:
                    if k[0] not in gff1.index or k[2] not in gff2.index:
                        continue
                    k[1], k[3] = gff1.loc[k[0], 'order'], gff2.loc[k[2], 'order']
                    newblock.append(k)
                if len(newblock) == 0:
                    continue
                collinearity.append([block[0], newblock, block[2]])
        elif '# Alignment' in p:
            collinearity = base.read_coliearity(self.collinearity)
        return collinearity



import os
import re
import shutil
import sys

import numpy as np
import pandas as pd
import wgdi.base as base


class colinearscan():
    def __init__(self, options):
        self.repeat_number = 20
        self.score = 100
        self.evalue = 1e-5
        self.position = 'order'
        self.blast_reverse = 'False'
        for k, v in options:
            setattr(self, str(k), v)
            print(str(k), ' = ', v)
        if hasattr(self, 'mg'):
            self.mg = self.mg.split(',')
        else:
            self.mg = [50, 50]
        if os.path.exists(self.dir):
            shutil.rmtree(self.dir)
        os.makedirs(self.dir)
        os.makedirs(self.dir+'/pair/')
        os.makedirs(self.dir+'/block/')

    def deal_blast(self, blast, gff1, gff2, repeat_number):
        index = [group.sort_values(by=11, ascending=False)[:repeat_number].index.tolist()
                 for name, group in blast.groupby([0])]
        index = np.array(index, dtype=object)
        if len(index):
            index = np.concatenate(index)
        else:
            index = []
        blast = blast.loc[index, [0, 1]]
        gff1 = gff1[['chr', 'stand', 'order']]
        gff2 = gff2[['chr', 'stand', 'order']]
        gff1.columns = ['chr1', 'stand1', 'loc1']
        gff2.columns = ['chr2', 'stand2', 'loc2']
        blast = pd.merge(blast, gff1, left_on=0, right_on=gff1.index)
        blast = pd.merge(blast, gff2, left_on=1, right_on=gff2.index)
        blast.replace({'+': '1', '-': '-1'}, inplace=True)
        return blast

    def rewriteblock(self, blast, file, fout):
        num = 0
        fout = open(fout, 'w')
        collinearity = base.read_colinearscan(file)
        for block in collinearity:
            num += 1
            if block[1][-1][0]+','+block[1][-1][2] not in blast.index:
                block[1] = block[1][:-1]
            fout.write('the '+str(num)+'th path length ' +
                       str(len(block[1]))+'\n')
            for k in block[1]:
                s = ' '.join([str(i) for i in k])
                fout.write(s+'\n')
            fout.write('>LOCALE p-value :'+block[2]+'\n\n')

    def run(self):
        lens1 = base.newlens(self.lens1, 'order')
        lens2 = base.newlens(self.lens2, 'order')
        lens1 = lens1[lens1 > 4]
        lens2 = lens2[lens2 > 4]
        gff1 = base.newgff(self.gff1)
        gff2 = base.newgff(self.gff2)
        gff1 = gff1[gff1['chr'].isin(lens1.index)]
        gff2 = gff2[gff2['chr'].isin(lens2.index)]
        blast = base.newblast(self.blast, int(self.score), float(
            self.evalue), gff1, gff2, self.blast_reverse)
        df = self.deal_blast(blast, gff1, gff2, int(self.repeat_number))
        files = []
        for (chr1, chr2), group in df.groupby(['chr1', 'chr2']):
            group = group.sort_values(by=['loc1', 'loc2'])
            dir1 = './'+self.dir+'/pair/'+str(chr1)+'.vs.'+str(chr2)+'.pair'
            dir2 = './'+self.dir+'/block/'+str(chr1)+'.vs.'+str(chr2)+'.blk'
            group[[0, 'stand1', 'loc1', 1, 'stand2', 'loc2']].to_csv(
                dir1, sep=' ', index=None, header=None)
            args = ['blockscan', '-chr1len', lens1[str(chr1)], '-chr2len', lens2[str(
                chr2)], '-mg1', self.mg[0], '-mg2', self.mg[1], dir1, '>'+dir2]
            command = ' '.join([str(k) for k in args])
            os.system(command)
            files.append(dir2)
        f = open(self.dir+'.block.old.txt', 'w')
        for k in files:
            fp = open(k)
            f.write(fp.read())
            fp.close()
        f.close()
        self.rewriteblock(blast, self.dir+'.block.old.txt',
                          self.dir+'.block.txt')
        shutil.rmtree(self.dir)
        sys.exit(0)



import argparse
import os
import sys

import pandas as pd
import wgdi
import wgdi.base as base
from wgdi.align_dotplot import align_dotplot
from wgdi.block_correspondence import block_correspondence
from wgdi.block_info import block_info
from wgdi.block_ks import block_ks
from wgdi.circos import circos
from wgdi.colinearscan import colinearscan
from wgdi.dotplot import dotplot
from wgdi.ks import ks
from wgdi.ks_peaks import kspeaks
from wgdi.peaksfit import peaksfit
from wgdi.ksfigure import ksfigure
from wgdi.retain import retain
from wgdi.pindex import pindex
from wgdi.trees import trees
from wgdi.run_colliearity import mycollinearity

parser = argparse.ArgumentParser(
    prog='wgdi', usage='%(prog)s [options]', epilog="", formatter_class=argparse.RawDescriptionHelpFormatter,)
parser.description = '''\
This is a gold standard for complex genomic analysis,including the construction 
of homologous gene dotplot, event-related genomic alignment, and synonymous 
substitutions, and differences in different evolution rates, etc.

    https://wgdi.readthedocs.io/en/latest/
    -------------------------------------- '''
parser.add_argument("-v", "--version", action='version', version='0.4.0')
parser.add_argument("-cl", dest="colinearscan",
                    help="A simple way to run ColinearScan")
parser.add_argument("-icl", dest="improvedcollinearity",
                    help="Improved version of ColinearScan ")
parser.add_argument("-ks", dest="calks",
                    help="Calculate Ka/Ks for homologous gene pairs by Comdel")
parser.add_argument("-bk", dest="blockks",
                    help="Show Ks of blocks in a dotplot")
parser.add_argument("-bi", dest="blockinfo",
                    help="Collinearity and Ks speculate whole genome duplication")
parser.add_argument("-d", dest="dotplot",
                    help="Show homologous gene dotplot")
parser.add_argument("-c", dest="correspondence",
                    help="Extract event-related genomic alignment")
parser.add_argument("-a", dest="alignment",
                    help="Show event-related genomic alignment in a dotplot")
parser.add_argument("-at", dest="alignmenttrees",
                    help="Collinear genes construct phylogenetic trees")
parser.add_argument("-p", dest="pindex",
                    help="Polyploidy-index characterize the degree of divergence between subgenomes of a polyploidy")
parser.add_argument("-r", dest="retain",
                    help="Show subgenomes in gene retention or genome fractionation")
parser.add_argument("-ci", dest="circos",
                    help="A simple way to run circos")
parser.add_argument("-kp", dest="kspeaks",
                    help="A simple way to get ks peaks")
parser.add_argument("-kf", dest="ksfigure",
                    help="A simple way to draw ks distribution map")
parser.add_argument("-pf", dest="peaksfit",
                    help="Gaussian fitting of ks distribution")
parser.add_argument("-conf", dest="configure",
                    help="Show all configured parameters")
args = parser.parse_args()


def run_dotplot():
    options = base.load_conf(args.dotplot, 'dotplot')
    dot = dotplot(options)
    dot.run()

def run_block_info():
    options = base.load_conf(args.blockinfo, 'blockinfo')
    blockinfo = block_info(options)
    blockinfo.run()


def run_circos():
    options = base.load_conf(args.circos, 'circos')
    cir = circos(options)
    cir.run()

def run_peaksfit():
    options = base.load_conf(args.peaksfit, 'peaksfit')
    pf = peaksfit(options)
    pf.run()

def run_align_dotplot():
    options = base.load_conf(args.alignment, 'alignment')
    align_dot = align_dotplot(options)
    align_dot.run()


def run_align_correspondence():
    options = base.load_conf(args.correspondence, 'correspondence')
    align_cor = block_correspondence(options)
    align_cor.run()


def run_retain():
    options = base.load_conf(args.retain, 'retain')
    retained = retain(options)
    retained.run()


def run_block_ks():
    options = base.load_conf(args.blockks, 'blockks')
    blockks = block_ks(options)
    blockks.run()

def run_kspeaks():
    options = base.load_conf(args.kspeaks, 'kspeaks')
    kp = kspeaks(options)
    kp.run()

def run_ksfigure():
    options = base.load_conf(args.ksfigure, 'ksfigure')
    kf = ksfigure(options)
    kf.run()

def run_pindex():
    options = base.load_conf(args.pindex, 'pindex')
    p = pindex(options)
    p.run()

def run_trees():
    options = base.load_conf(args.alignmenttrees, 'alignmenttrees')
    t = trees(options)
    t.run()


def run_cal_ks():
    options = base.load_conf(args.calks, 'ks')
    calks = ks(options)
    calks.run()


def run_colinearscan():
    options = base.load_conf(args.colinearscan, 'colinearscan')
    col = colinearscan(options)
    col.run()

def run_collinearity():
    options = base.load_conf(args.improvedcollinearity, 'collinearity')
    col = mycollinearity(options)
    col.run()


def run_configure():
    options = base.load_conf(args.conf, 'total')

def module_to_run(argument):
    switcher = {
        'dotplot': run_dotplot,
        'correspondence': run_align_correspondence,
        'alignment': run_align_dotplot,
        'retain': run_retain,
        'blockks': run_block_ks,
        'blockinfo': run_block_info,
        'calks': run_cal_ks,
        'colinearscan': run_colinearscan,
        'circos': run_circos,
        'kspeaks': run_kspeaks,
        'peaksfit':run_peaksfit,
        'ksfigure': run_ksfigure,
        'pindex':run_pindex,
        'alignmenttrees':run_trees,
        'improvedcollinearity':run_collinearity,
        'conf':run_configure,
    }
    return switcher.get(argument)()


def main():
    path = wgdi.__path__[0]
    options = {'dotplot': 'dotplot.conf',
               'correspondence': 'corr.conf',
               'alignment': 'align.conf',
               'retain': 'retain.conf',
               'blockks': 'blockks.conf',
               'blockinfo': 'blockinfo.conf',
               'calks': 'ks.conf',
               'colinearscan': 'colinearscan.conf',
               'circos': 'circos.conf',
               'kspeaks': 'kspeaks.conf',
               'ksfigure': 'ksfigure',
               'pindex':'pindex.conf',
               'alignmenttrees':'alignmenttrees.conf',
               'peaksfit':'peaksfit.conf',
               'configure':'total.conf',
               'improvedcollinearity':'collinearity.conf',
               }
    for arg in vars(args):
        value = getattr(args, arg)
        if value is not None:
            if value in ['?', 'help', 'example']:
                f = open(os.path.join(path, 'example', options[arg]))
                print(f.read())
            elif not os.path.exists(value):
                print(value+' not exits')
                sys.exit(0)
            else:
                module_to_run(arg)
